<?php

define('SYNO_PHOTO_EXIV2_PROG', '/usr/syno/bin/exiv2');
define('SYNO_PHOTO_DSM_USER_PROG', '/usr/syno/bin/synophoto_dsm_user');

/* list dsm users */
$dsmUserList = getDSMUserList();
$usernameList = array_merge(array(''), $dsmUserList);

foreach ($usernameList as $username) {
	$user = $username;
	$dbh = getDatabaseHandler();
	$escapeStr = getEscapeString();

	//no database, skip
	if (!$dbh) {
		continue;
	}

	$photoList = getPhotoListFromDB();

	foreach ($photoList as $photo) {
		//read tags from db
		$dbList = readTagListFromDB($photo['id']);

		//read tags from metadata
		$metadataList = readTagListFromMetadata($photo['path']);

		//rename db tag if the metadata tag with the same name exist
		renameDuplicateTagInDBList($dbList, $metadataList);

		//write tags
		writeTagListToMetadata($photo, $dbList, $metadataList);
	}
}

@exec('/usr/syno/bin/synosetkeyvalue /var/packages/PhotoStation/etc/settings.conf ms_people_tag_merged yes');

foreach ($usernameList as $username) {
	if (strlen($username)){
		$user_photo_db = "/var/services/homes/$username/photo/.SYNOPPSDB";

		if(file_exists($user_photo_db)){
			@exec("/usr/syno/bin/synoindex -R /var/services/homes/$username/photo");
		}
	} else {
		@exec("/usr/syno/bin/synoindex -R photo");
	}
}

function getDSMUserList()
{
	@exec(SYNO_PHOTO_DSM_USER_PROG." --enum", $dsmUserCount);
	@exec(SYNO_PHOTO_DSM_USER_PROG." --enum 0 ".intval($dsmUserCount[0]), $dsmUserList);

	foreach($dsmUserList as &$dsmUser) {
		$dsmUser = explode(',', $dsmUser);
		$dsmUser = $dsmUser[1];
	}

	return $dsmUserList;
}

function getDatabaseHandler()
{
	global $user;

	$dbh = null;

	if (strlen($user)){
		$user_photo_db = "/var/services/homes/$user/photo/.SYNOPPSDB";

		if(!file_exists($user_photo_db)){
			return $dbh;
		}

		try {
			$dbh = new PDO("sqlite:{$user_photo_db}");
			$dbh->exec('PRAGMA case_sensitive_like=1');
			$dbh->exec('PRAGMA foreign_keys=ON');
		} catch (PDOException $e) {
			die($e->getMessage());
		}
	} else {
		try {
			$dbh = new PDO('pgsql:dbname=photo', 'admin', '', array(PDO::ATTR_PERSISTENT => false, PDO::ATTR_EMULATE_PREPARES => true));
		} catch (PDOException $e) {
			die($e->getMessage());
		}
	}

	return $dbh;
}

function getEscapeString()
{
	global $user;

	return strlen($user) ? "ESCAPE '\'" : "";
}

function getPhotoListFromDB()
{
	global $dbh, $escapeStr;

	$photoList = array();

	$sqlCond = 'SELECT id, path, resolutionx, resolutiony FROM photo_image WHERE id IN (SELECT DISTINCT image_id FROM photo_image_label)';
	$sqlParam = array();

	$dbResult = $dbh->prepare($sqlCond);
	$dbResult->execute($sqlParam);

	while (false !== ($row = $dbResult->fetch())) {
		if (!inAcceptableExtList($row['path'])) {
			continue;
		}

		$photoList[] = array(
			'id' => $row['id'],
			'path' => $row['path'],
			'resolutionx' => $row['resolutionx'],
			'resolutiony' => $row['resolutiony']
		);
	}

	return $photoList;
}

function inAcceptableExtList($path)
{
	$acceptableExtList = array('jpg', 'jpeg', 'jpe', 'tif', 'tiff', 'png', 'bmp');

	foreach ($acceptableExtList as $acceptableExt) {
		if (0 === substr_compare(strtolower($path), $acceptableExt, strlen($path) - strlen($acceptableExt), strlen($acceptableExt))) {
			return true;
		}
	}

	return false;
}

function readTagListFromDB($photoId)
{
	global $dbh, $escapeStr;

	$tagList = array();
	$nameList = array();

	$sqlCond =
		'SELECT photo_label.id, photo_label.name, photo_image_label.info FROM photo_image_label '.
		'INNER JOIN photo_label ON photo_image_label.label_id = photo_label.id '.
		'WHERE photo_label.category = 0 AND photo_image_label.status = \'t\' AND photo_image_label.image_id = '.$photoId;
	$sqlParam = array();

	$dbResult = $dbh->prepare($sqlCond);
	$dbResult->execute($sqlParam);

	while (false !== ($row = $dbResult->fetch())) {
		$tagList[] = array(
			'id' => $row['id'],
			'name' => $row['name'],
			'info' => json_decode($row['info'], true)
		);

		$nameList[$row['name']] = $row['name'];
	}

	return array(
		'tagList' => $tagList,
		'nameList' => $nameList
	);
}

function readTagListFromMetadata($path)
{
	global $user;

	$tagList = array();
	$nameList = array();
	$peopleTagPrefix = 'Xmp.MP.RegionInfo/MPRI:Regions[';
	$tagIndex = 1;

	$filePath = $path;
	if ('' !== $user) {
		$filePath = "/var/services/homes/$user/photo/".$filePath;
	}

	$cmd = SYNO_PHOTO_EXIV2_PROG.' -Pkt '.escapeshellarg($filePath);
	@exec($cmd, $cmdResult);

	foreach ($cmdResult as $tag) {
		if ($peopleTagPrefix !== substr($tag, 0, strlen($peopleTagPrefix))) {
			continue;
		}

		$targetTagPrefix = $peopleTagPrefix.$tagIndex.']';
		$nextTagPrefix = $peopleTagPrefix.($tagIndex + 1).']';

		if ($nextTagPrefix === substr($tag, 0, strlen($nextTagPrefix))) {
			//already arrive next people tag
			$tagIndex++;
			continue;
		}

		$subTag = explode(' ', $tag);
		$key = $subTag[0];
		$value = trim(substr($tag, strlen($key), strlen($tag) - strlen($key)));

		if ($key === $targetTagPrefix.'/MPReg:PersonDisplayName') {
			$tagList[$tagIndex]['PersonDisplayName'] = $value;
			$nameList[$value] = $value;
		} else if ($key === $targetTagPrefix.'/MPReg:Rectangle') {
			$tagList[$tagIndex]['Rectangle'] = $value;
		}
	}

	return array(
		'tagList' => $tagList,
		'nameList' => $nameList
	);
}

function renameDuplicateTagInDBList(&$dbList, $metadataList)
{
	foreach ($dbList['nameList'] as $key => $name) {
		$suffix = 1;
		$newName = $name;

		while (true) {
			if (isset($metadataList['nameList'][$newName])) {
				$newName = $name.'_'.($suffix++);
				continue;
			}

			$arrayKeys = array_keys($dbList['nameList'], $newName);
			$arrayKeysCount = count($arrayKeys);

			if (!$arrayKeysCount || (1 == count($arrayKeys) && $key === $arrayKeys[0])) {
				//no same tag name in current dbNameList || only one same tag name and it is this tag name itself
				break;
			}

			$newName = $name.'_'.($suffix++);
		}

		$dbList['nameList'][$key] = $newName;
	}
}

function writeTagListToMetadata($photo, $dbList, $metadataList)
{
	$dbTagList = $dbList['tagList'];
	$dbNameList = $dbList['nameList'];
	$metadataTagCount = count($metadataList['tagList']);

	//insert new tags from DB into metadata
	foreach ($dbTagList as $dbTag) {
		if (!strlen($dbNameList[$dbTag['name']]) || !($rectangle = convertDBJsonToMetadataRectangle($photo, $dbTag['info']))) {
			continue;
		}

		insertTagIntoMetadata($photo, ++$metadataTagCount, $dbNameList[$dbTag['name']], $rectangle);
	}
}

function insertTagIntoMetadata($photo, $index, $newName, $rectangle)
{
	global $user;

	$nameSpaceCmd = '-M"reg MP http://ns.microsoft.com/photo/1.2/" '.
		'-M"reg MPRI http://ns.microsoft.com/photo/1.2/t/RegionInfo#" '.
		'-M"reg MPReg http://ns.microsoft.com/photo/1.2/t/Region#"';

	$filePath = $photo['path'];
	if ('' !== $user) {
		$filePath = "/var/services/homes/$user/photo/".$filePath;
	}

	if (1 === $index) {
		$cmd = sprintf('%s %s %s %s', SYNO_PHOTO_EXIV2_PROG, $nameSpaceCmd,
					   '-M"set Xmp.MP.RegionInfo/MPRI:Regions XmpText type=Bag"',
					   escapeshellarg($filePath));
		@exec($cmd);
	}

	$cmd = sprintf('%s %s %s %s %s', SYNO_PHOTO_EXIV2_PROG, $nameSpaceCmd,
				   '-M"set Xmp.MP.RegionInfo/MPRI:Regions['.$index.']/MPReg:Rectangle '.$rectangle.'"',
				   '-M"set Xmp.MP.RegionInfo/MPRI:Regions['.$index.']/MPReg:PersonDisplayName '.str_replace('"', '\"', $newName).'"',
				   escapeshellarg($filePath));
	@exec($cmd);
}

function convertDBJsonToMetadataRectangle($photo, $info)
{
	$precision = 6;
	$thumbSize = 800;

	$ratio = floatval($photo['resolutiony']) / floatval($photo['resolutionx']);

	if ($ratio > 1) {//higher image
		$x = round(floatval($info['x']) / ($thumbSize / $ratio), $precision);
		$y = round(floatval($info['y']) / $thumbSize, $precision);
		$width = round(floatval($info['width']) / ($thumbSize / $ratio), $precision);
		$height = round(floatval($info['height']) / $thumbSize, $precision);
	} else {//wider image
		$x = round(floatval($info['x']) / $thumbSize, $precision);
		$y = round(floatval($info['y']) / ($thumbSize * $ratio), $precision);
		$width = round(floatval($info['width']) / $thumbSize, $precision);
		$height = round(floatval($info['height']) / ($thumbSize * $ratio), $precision);
	}

	//prevent 0 for width & height
	$width = $width > 0 ? $width : 0.01;
	$height = $height > 0 ? $height : 0.01;

	return "$x, $y, $width, $height";
}

?>
